#!/bin/bash

echo -ne "FuseJS: building distribution from sources"

# set build infos
VERSION="0"
REVISION="1"
PLATFORM=`uname -s`

# set used variables
BASEDIR=`pwd`
PKGNAME="fuse"
RELEASE=`date +%Y%m%d`

# set your sourcedir here
SOURCES=~/Work/fusejs

if [ $PLATFORM == 'Darwin' ]; then
  JSMIN=./bin/jsmin_macos
elif [ $PLATFORM == 'Linux' ]; then
  JSMIN=./bin/jsmin_linux
elif [ $PLATFORM == 'Windows' ]; then
  JSMIN=./bin/jsmin.exe
fi

minimize() {

  > dist/$PKGNAME.js
  > dist/$PKGNAME-min.js

  # add opening closure to source file
  cat bin/HEAD |\
    sed "s/@FUSEJS_DATE/${RELEASE}/g" |\
    sed "s/@FUSEJS_VERSION/${VERSION}/g" |\
    sed "s/@FUSEJS_REVISION/${REVISION}/g" >> dist/$PKGNAME.js
  cat bin/INTRO >> dist/$PKGNAME.js
  echo >> dist/$PKGNAME.js

  # add opening closure to minified file
  cat bin/HEAD |\
    sed "s/@FUSEJS_DATE/${RELEASE}/g" |\
    sed "s/@FUSEJS_VERSION/${VERSION}/g" |\
    sed "s/@FUSEJS_REVISION/${REVISION}/g" >> dist/$PKGNAME-min.js
  cat bin/INTRO >> dist/$PKGNAME-min.js

  # now loop through all components
  for (( i = 0 ; i < ${#components[@]}; i++))
  do {

    # log line count and name
    #wc -l src/${components[$i]}

    # add components to source file
    cat src/${components[$i]} >> dist/$PKGNAME.js
    echo >> dist/$PKGNAME.js

	# create temporary minified component
    sed -e "s/@FUSEJS_VERSION/${VERSION}.${REVISION}/g" \
	  src/${components[$i]} | $JSMIN | tr -d "\n" >> dist/$PKGNAME-min.js

    echo -ne "."

  } done

  # add end of main closure in source file
  cat bin/OUTRO >> dist/$PKGNAME.js
  echo >> dist/$PKGNAME.js

  # add end of main closure in minified file
  tr -d [:space:] < bin/OUTRO >> dist/$PKGNAME-min.js

  # build a compressed version of the minified file 
  #gzip -c -n9 dist/$PKGNAME-min.js > dist/$PKGNAME.jsz

  # build a versioned file name of the minified file
  #cp dist/$PKGNAME-min.js dist/$PKGNAME-$VERSION.$REVISION.js
}

# fuse components
components=(

  fuse-c.js \
  browser.js \
  features.js \
  fusebox.js \

  lang/object.js \
  lang/class.js \
  lang/function.js \
  lang/enumerable.js \
  lang/array.js \
  lang/number.js \
  lang/regexp.js \
  lang/string.js \
  lang/hash.js \
  lang/range.js \
  lang/template.js \
  lang/timer.js \
  lang/json.js \

  ajax/ajax.js \
  ajax/responders.js \
  ajax/base.js \
  ajax/request.js \
  ajax/updater.js \
  ajax/timed-updater.js \

  dom/element/element.js \
  dom/element/attribute.js \
  dom/element/style.js \
  dom/element/traversal.js \
  dom/element/position.js \
  dom/element/helpers.js \

  dom/form/form.js \
  dom/form/timed-observer.js \
  dom/form/event-observer.js \

  dom/node.js \
  dom/viewport.js \

  dom/selector/nwmatcher.js \

  dom/event/event.js \
  dom/event/dom-loaded.js \

)

pushd . 2&> /dev/null

cd $SOURCES

minimize $components

cat vendor/nwbox/nwmatcher/src/nwmatcher.js \
  >> dist/$PKGNAME.js

cat vendor/nwbox/nwmatcher/src/nwmatcher.js \
  | $JSMIN | tr -d "\n" >> dist/$PKGNAME-min.js

echo ""

popd 2&> /dev/null

exit 0
